package org.jvnet.hifaces.components.templatedpanel.impl;

import java.io.IOException;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheException;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jvnet.hifaces.components.templatedpanel.Template;
import org.jvnet.hifaces.components.templatedpanel.TemplateProvider;
import org.jvnet.hifaces.util.DualString;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class DefaultTemplateProvider implements TemplateProvider {

	private static final String FRAGMENT_ID = "fragmentId"; //$NON-NLS-1$

	private final Log logger = LogFactory.getLog(getClass());

	public static final String DEFAULT_USE_FACET_ATTRIBUTE_NAME = "use-facet";

	private static final String DEFAULT_CACHE_NAME = "templatesCache";

	private CacheManager cacheManager = null;

	public CacheManager getCacheManager() {
		if (cacheManager == null) {
			cacheManager = createDefaultCacheManager();
		}
		return cacheManager;
	}

	protected CacheManager createDefaultCacheManager() {
		try {
			return CacheManager.getInstance();
		} catch (CacheException cacheException) {
			throw new AssertionError(cacheException);
		}
	}

	public void setCacheManager(CacheManager cacheManager) {
		this.cacheManager = cacheManager;
	}

	public String cacheName = DEFAULT_CACHE_NAME;

	public String getCacheName() {
		return cacheName;
	}

	public void setCacheName(String cacheName) {
		this.cacheName = cacheName;
	}

	private Cache cache;

	public Cache getCache() {
		if (cache != null) {
			return cache;
		} else {
			final CacheManager cacheManager = getCacheManager();
			final String cacheName = getCacheName();
			if (cacheManager.cacheExists(cacheName)) {
				return cacheManager.getCache(cacheName);
			} else {
				cache = createDefaultCache(cacheManager, cacheName);
				return cache;
			}
		}

		// if (!cacheManager.cacheExists(cacheName)) {
		// try {
		// final Cache cache = ;
		// this.cache = cacheManager.getCache(cacheName);
		// }
		// catch (CacheException cacheException) {
		// throw new ExceptionInInitializerError(cacheException);
		// }
		// }
		// }
	}

	protected Cache createDefaultCache(final CacheManager cacheManager,
			final String cacheName) {
		try {
			final Cache cache = new Cache(cacheName, 8192, true, true, 0, 0);
			cacheManager.addCache(cache);
			return cache;
		} catch (CacheException ex) {
			throw new AssertionError(ex);
		}
	}

	public void setCache(Cache cache) {
		this.cache = cache;
	}

	private String useFacetAttributeName = DEFAULT_USE_FACET_ATTRIBUTE_NAME;

	public void setUseFacetAttributeName(String useFacetAttributeName) {
		this.useFacetAttributeName = useFacetAttributeName;
	}

	private TemplateNameResolverFactory templateNameResolverFactory;

	public TemplateNameResolverFactory getTemplateNameResolverFactory() {
		return templateNameResolverFactory;
	}

	public void setTemplateNameResolverFactory(
			TemplateNameResolverFactory templateNameResolverFactory) {
		this.templateNameResolverFactory = templateNameResolverFactory;
	}

	private TemplateNameResolver templateNameResolver;

	public TemplateNameResolver getTemplateNameResolver() {
		return templateNameResolver;
	}

	public void setTemplateNameResolver(
			TemplateNameResolver templateNameResolver) {
		this.templateNameResolver = templateNameResolver;
	}

	public boolean providesTemplate(FacesContext contex, String templateName,
			String fragmentId) {
		try {
			final Template template = getTemplate(contex, templateName,
					fragmentId);
			return template != null;
		} catch (Exception ex) {
			return false;
		}
	}

	public Template getTemplate(FacesContext context, String templateName,
			String fragmentId) throws Exception {

		final TemplateResource templateFile = getTemplateResource(context, templateName);
		final String path = templateFile.getKey();
		final DualString documentKey = new DualString(path, null);

		final Element cacheDocument = getCache().get(documentKey);
		if (cacheDocument == null
				|| cacheDocument.getCreationTime() < templateFile
						.getLastModified()) {
			final org.w3c.dom.Element templateRootElement = templateFile
					.getAsDomElement();
			synchronized (getCache()) {
				processDocument(path, templateRootElement);
			}
		}
		final DualString fragmentKey = new DualString(path, fragmentId);
		final Element cacheFragment = getCache().get(fragmentKey);

		if (cacheFragment == null) {
			throw new Exception("Could not load template with name ["
					+ templateName + "] and fragment id [" + fragmentId + "].");
		}
		return ((Template) cacheFragment.getValue());
	}

	private TemplateResource getTemplateResource(FacesContext context, String templateName) throws IOException {
		final TemplateNameResolver templateNameResolver = getTemplateNameResolver(context);
		final TemplateResource templateFile = templateNameResolver.resolve(
				context, templateName);
		return templateFile;
	}

	private TemplateNameResolver getTemplateNameResolver(FacesContext context) {
		final TemplateNameResolver templateNameResolver;
		final TemplateNameResolver providedTemplateNameResolver = getTemplateNameResolver();
		if (providedTemplateNameResolver != null) {
			templateNameResolver = providedTemplateNameResolver;

		} else {
			final TemplateNameResolverFactory templateNameResolverFactory = getTemplateNameResolverFactory(context);
			templateNameResolver = templateNameResolverFactory
					.getTemplateNameResolver(context);

		}
		return templateNameResolver;
	}

	private TemplateNameResolverFactory getTemplateNameResolverFactory(FacesContext context) {
		final TemplateNameResolverFactory templateNameResolverFactory;
		final TemplateNameResolverFactory providedTemplateNameResolverFactory = getTemplateNameResolverFactory();
		if (providedTemplateNameResolverFactory != null) {
			templateNameResolverFactory = providedTemplateNameResolverFactory;
		} else {
			templateNameResolverFactory = DefaultTemplateNameResolverFactory
					.getInstance(context);
		}
		return templateNameResolverFactory;
	}

	private void processDocument(String path,
			org.w3c.dom.Element templateRootElement) throws Exception {
		final List keys = getCache().getKeys();
		final DualString[] keyArray = (DualString[]) keys
				.toArray(new DualString[keys.size()]);
		for (int i = 0; i < keyArray.length; i++) {
			if (keyArray[i].getHi().equals(path)) {
				getCache().remove(keyArray[i]);
			}
		}
		final DualString key = new DualString(path, null);
		final CompiledDomElementTemplate template = createTemplate(path, null,
				(org.w3c.dom.Element) templateRootElement.cloneNode(true));
		getCache().put(new Element(key, template));
		// fragmente rekursiv einfgen
		processElement(path, templateRootElement);
	}

	private void processElement(String path, org.w3c.dom.Element element)
			throws Exception {
		final String fragmentId = element.getAttribute(FRAGMENT_ID);
		if (!(fragmentId == null || "".equals(fragmentId))) {
			final DualString key = new DualString(path, fragmentId);
			// final SerializableDomElement value = new
			// SerializableDomElement((org.w3c.dom.Element) element
			// .cloneNode(true));
			final CompiledDomElementTemplate template = createTemplate(path,
					fragmentId, (org.w3c.dom.Element) element.cloneNode(true));
			getCache().put(new Element(key, template));
		}
		final NodeList childNodes = element.getChildNodes();
		for (int i = 0; i < childNodes.getLength(); i++) {
			final Node item = childNodes.item(i);
			if (item.getNodeType() == Node.ELEMENT_NODE) {
				processElement(path, (org.w3c.dom.Element) item);
			}
		}

	}

	private CompiledDomElementTemplate createTemplate(String path,
			final String fragmentId, org.w3c.dom.Element element) {
		return new CompiledDomElementTemplate(path, fragmentId, element,
				useFacetAttributeName);
	}

}