package org.jvnet.hifaces.servlet.http;

import java.io.IOException;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jvnet.hifaces.counter.Counter;
import org.springframework.web.filter.GenericFilterBean;

public class RequestTrackingFilter extends GenericFilterBean {

	public Log logger = LogFactory.getLog(getClass());

	public static final String REQUEST_COUNTER_SESSION_ATTRIBUTE_NAME = "_hiRequestCounter";

	public static final String REQUEST_ID_REQUEST_PARAMETER_NAME = "hirid";

	public static final String CURRENT_REQUEST_ID_REQUEST_ATTRIBUTE_NAME = "previousRequestId";

	public static final String PREVIOUS_REQUEST_ID_REQUEST_ATTRIBUTE_NAME = "currentRequestId";

	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {

		if (request instanceof HttpServletRequest
				&& response instanceof HttpServletResponse) {
			doHttpFilter((HttpServletRequest) request,
					(HttpServletResponse) response, chain);
		} else {
			chain.doFilter(request, response);
		}
	}

	public void doHttpFilter(HttpServletRequest request,
			HttpServletResponse response, FilterChain chain)
			throws IOException, ServletException {

		final String requestIdParameter = request
				.getParameter(REQUEST_ID_REQUEST_PARAMETER_NAME);
		if (requestIdParameter != null) {
			try {
				final Long previousRequestId = Long
						.parseLong(requestIdParameter);
				request.setAttribute(
						PREVIOUS_REQUEST_ID_REQUEST_ATTRIBUTE_NAME,
						previousRequestId);
			} catch (NumberFormatException nfe) {
				logger.warn("Could not parse request id [" + requestIdParameter
						+ "].");
			}
		}

		final Counter requestCounter;
		{
			final HttpSession session = request.getSession();
			final Object draftRequestCounter = session
					.getAttribute(RequestTrackingFilter.REQUEST_COUNTER_SESSION_ATTRIBUTE_NAME);
			if (draftRequestCounter instanceof Counter) {
				requestCounter = (Counter) draftRequestCounter;
			} else {
				requestCounter = new Counter();
				session
						.setAttribute(
								RequestTrackingFilter.REQUEST_COUNTER_SESSION_ATTRIBUTE_NAME,
								requestCounter);

			}

			final Long currentRequestId = requestCounter.nextId();
			request.setAttribute(CURRENT_REQUEST_ID_REQUEST_ATTRIBUTE_NAME,
					currentRequestId);

			final HttpServletResponse wrappedResponse = new HttpServletResponseWrapper(
					response) {
				@Override
				public void sendRedirect(String location) throws IOException {

					final String newLocation;

					if (location.contains("?")) {
						newLocation = location + "&"
								+ REQUEST_ID_REQUEST_PARAMETER_NAME + "="
								+ currentRequestId;
					} else {
						newLocation = location + "?"
								+ REQUEST_ID_REQUEST_PARAMETER_NAME + "="
								+ currentRequestId;
					}
					super.sendRedirect(newLocation);
				}
			};
			chain.doFilter(request, wrappedResponse);
		}
	}
}
