package org.jvnet.hifaces.text;

import java.util.ResourceBundle;

import javax.faces.context.FacesContext;

/**
 * Singular text value binding. Text is resolved by a single message id.
 * 
 * @author valikov
 * 
 */

public abstract class AbstractSingularText extends AbstractText {
	@Override
	protected String getText(FacesContext facesContext,
			ResourceBundle resourceBundle) throws Exception {
		final String messageId = getMessageId(facesContext);
		final Object[] arguments = getArguments(facesContext);
		return getText(facesContext, resourceBundle, messageId, arguments);
	}

	/**
	 * Returns the id of the message,
	 * 
	 * @param facesContext
	 *            Faces context.
	 * @return Id of the message.
	 */
	protected abstract String getMessageId(final FacesContext facesContext);

	/**
	 * Returns message formatting arguments. May be <code>null</code>
	 * 
	 * @param facesContext
	 *            Faces context
	 * @return Message formatting arguments.
	 */
	protected Object[] getArguments(FacesContext facesContext) {
		return null;
	}

}
