package org.jvnet.hifaces.util;

import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;

import org.apache.commons.lang.Validate;

public class IdUtils {

	private IdUtils() {

	}

	/**
	 * Assigns id to a component. Uses <code>"hi"</code> as prefix.
	 * 
	 * @param component
	 *            Target component.
	 */
	public static void assignId(UIComponent component) {
		IdUtils.assignId("h", component);
	}

	/**
	 * Assigns id to the component. Uses
	 * <code>FacesContext.getCurrentInstance()</code> as context. If there's
	 * no current context, does nothing.
	 * 
	 * @param prefix
	 *            id prefix.
	 * @param component
	 *            target component.
	 */
	public static void assignId(String prefix, UIComponent component) {
		final FacesContext context = FacesContext.getCurrentInstance();
		if (context != null) {
			assignId(context, prefix, component);
		}
	}

	/**
	 * Assigns id to the component. Generates a new id using the current view
	 * root. If there's no current view root, does nothing.
	 * 
	 * @param context
	 *            context.
	 * @param prefix
	 *            id prefix.
	 * @param component
	 *            target component.
	 */
	public static void assignId(final FacesContext context, String prefix,
			UIComponent component) {
		Validate.notNull(context, "Context must not be null.");
		Validate.notNull(prefix, "Prefix must not be null.");
		Validate.notNull(component, "Component must not be null.");
		final UIViewRoot viewRoot = context.getViewRoot();
		if (viewRoot != null) {
			final String id = viewRoot.createUniqueId();
			component.setId(prefix + id);
		}
	}
}
