package org.jvnet.hifaces.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.faces.component.UICommand;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;

import org.apache.myfaces.custom.navmenu.UINavigationMenuItem;
import org.apache.myfaces.custom.navmenu.jscookmenu.HtmlCommandJSCookMenu;
import org.jvnet.hifaces.binding.StaticStringMethodBinding;
import org.jvnet.hifaces.text.Text;

public class MenuUtils {

	public static HtmlCommandJSCookMenu createJSCookMenu(
			UINavigationMenuItem[][] menuItemArrays) {
		return createJSCookMenu(concat(menuItemArrays));
	}

	public static HtmlCommandJSCookMenu createJSCookMenu(
			UINavigationMenuItem[] menuItems) {
		final HtmlCommandJSCookMenu menu = new HtmlCommandJSCookMenu();
		IdUtils.assignId(menu);
		menu.setRendererType("org.apache.myfaces.JSCookMenu");
		// menu.getAttributes().put(JSFAttr.JAVASCRIPT_LOCATION,
		// "/pi/script/jscookmenu");
		// menu.getAttributes().put(JSFAttr.IMAGE_LOCATION,
		// "/pi/images/jscookmenu");
		// menu.getAttributes().put(JSFAttr.STYLE_LOCATION,
		// "/pi/style/jscookmenu");
		menu.setLayout("hbr");
		menu.setTheme("ThemeOffice");
		for (int index = 0; index < menuItems.length; index++) {
			final UINavigationMenuItem item = menuItems[index];
			menu.getChildren().add(item);
		}
		return menu;
	}

	public static UINavigationMenuItem createNavigationMenuItem(
			String messageId, MethodBinding action) {
		final UINavigationMenuItem menuItem = createNavigationMenuItem();
		menuItem.setValueBinding("itemLabel", new Text(messageId));
		if (action != null)
			menuItem.setAction(action);
		menuItem.setId(FacesContext.getCurrentInstance().getViewRoot()
				.createUniqueId());
		return menuItem;
	}

	public static UINavigationMenuItem createNavigationMenuItem() {
		final UINavigationMenuItem menuItem = new UINavigationMenuItem();
		IdUtils.assignId(menuItem);
		return menuItem;
	}

	public static UINavigationMenuItem createNavigationMenuItem(
			String messageId, UINavigationMenuItem... childItems) {
		final UINavigationMenuItem menuItem = createNavigationMenuItem();
		menuItem.setValueBinding("itemLabel", new Text(messageId));
		if (childItems != null) {
			for (int index = 0; index < childItems.length; index++) {
				final UINavigationMenuItem childItem = childItems[index];
				menuItem.getChildren().add(childItem);
			}
		}
		return menuItem;
	}

	public static UINavigationMenuItem createNavigationMenuItem(
			String messageId, MethodBinding action,
			UINavigationMenuItem... childItems) {
		final UINavigationMenuItem menuItem = createNavigationMenuItem(
				messageId, childItems);
		if (action != null)
			menuItem.setAction(action);
		return menuItem;
	}

	public static UINavigationMenuItem createNavigationMenuItem(
			String messageId, String target, UINavigationMenuItem... childItems) {
		final MethodBinding action = new StaticStringMethodBinding(target);
		return createNavigationMenuItem(messageId, action, childItems);
	}

	public static UINavigationMenuItem createNavigationMenuItem(
			String messageId, String target) {
		final MethodBinding action = new StaticStringMethodBinding(target);
		return createNavigationMenuItem(messageId, action);
	}

	public static UINavigationMenuItem[] concat(
			UINavigationMenuItem[]... menuItemArrays) {
		final List list = new ArrayList();
		for (int index = 0; index < menuItemArrays.length; index++) {
			final UINavigationMenuItem[] items = menuItemArrays[index];
			for (int jndex = 0; jndex < items.length; jndex++) {
				final UINavigationMenuItem item = items[jndex];
				list.add(item);
			}
		}
		return toArray(list);
	}

	public static UINavigationMenuItem[] toArray(
			Collection<UINavigationMenuItem> collection) {
		if (collection == null)
			return null;
		else
			return (UINavigationMenuItem[]) collection
					.toArray(new UINavigationMenuItem[collection.size()]);
	}

	public static UINavigationMenuItem createNavigationMenuItem(
			UICommand command) {
		final UINavigationMenuItem item = createNavigationMenuItem();

		if (command.getValueBinding("value") != null) {
			item.setValueBinding("itemLabel", command.getValueBinding("value"));
		}
		if (command.getValueBinding("rendered") != null) {
			item.setValueBinding("rendered", command
					.getValueBinding("rendered"));
		}

		if (command.getValue() instanceof String) {
			item.setItemLabel((String) command.getValue());
		}

		if (command.getAction() != null) {
			item.setAction(command.getAction());
		}
		return item;

	}
}
